package com.mmoragues.herois.di

import android.app.Application
import androidx.room.Room
import com.mmoragues.herois.data.database.HeroDatabase
import com.mmoragues.herois.data.database.LocalDataSource
import com.mmoragues.herois.data.database.LocalDataSourceImpl
import com.mmoragues.herois.data.remote.FakeRemoteDataSourceImplUITest
import com.mmoragues.herois.data.remote.RemoteDataSource
import com.mmoragues.herois.data.repository.HeroRepositoryImpl
import com.mmoragues.herois.domain.repository.HeroRepository
import com.mmoragues.herois.domain.useCase.*
import dagger.Module
import dagger.Provides
import dagger.hilt.InstallIn
import dagger.hilt.components.SingletonComponent
import javax.inject.Singleton

/**
 * Mòdul de Hilt des d'on es proveeix les diferents dependències pels Test
 */
@Module
@InstallIn(SingletonComponent::class)
object TestAppModule {

    //Provider de la BBDD Room en Memòria i la prepropulem amb dades per fer les proves
    @Provides
    @Singleton
    fun providerHeroDataBase(app: Application): HeroDatabase {
        return Room.databaseBuilder(
            app,
            HeroDatabase::class.java,
            "TESTDB"
        ).createFromAsset("database/heros.db")
            .build()
    }

    //Provider LocalDataSource
    @Provides
    @Singleton
    fun providerLocalDataSource(db: HeroDatabase): LocalDataSource {
        return LocalDataSourceImpl(db)
    }

    //Provider RemoteDataSource
    @Provides
    @Singleton
    fun providerRemoteDataSource(): RemoteDataSource {
        return FakeRemoteDataSourceImplUITest()
    }

    //Provider HeroRepository
    @Provides
    @Singleton
    fun providerHeroRepository(
        localDataSource: LocalDataSource,
        remoteDataSource: RemoteDataSource
    ): HeroRepository {
        return HeroRepositoryImpl(localDataSource, remoteDataSource)
    }

    //Provider GetAllHeroesUseCase
    @Provides
    @Singleton
    fun providerGetAllHeroesUseCase(repository: HeroRepository): GetAllHeroesUseCase {
        return GetAllHeroesUseCase(repository)
    }

    //Provider GetFavoritesHeroesUseCase
    @Provides
    @Singleton
    fun providerGetFavoritesHeroesUseCase(repository: HeroRepository): GetFavoritesHeroesUseCase {
        return GetFavoritesHeroesUseCase(repository)
    }

    //Provider GetHeroByIdUseCase
    @Provides
    @Singleton
    fun providerGetHeroByIdUseCase(repository: HeroRepository): GetHeroByIdUseCase {
        return GetHeroByIdUseCase(repository)
    }

    //Provider GetRandomHeroUseCase
    @Provides
    @Singleton
    fun providerGetRandomHeroUseCase(repository: HeroRepository): GetRandomHeroUseCase {
        return GetRandomHeroUseCase(repository)
    }

    //Provider ToggleFavoriteUseCase
    @Provides
    @Singleton
    fun providerToggleFavoriteUseCase(repository: HeroRepository): ToggleFavoriteUseCase {
        return ToggleFavoriteUseCase(repository)
    }
}


