package com.mmoragues.herois.ui.screens.favoritesScreen

import android.content.Context
import androidx.compose.ui.test.*
import androidx.compose.ui.test.junit4.createAndroidComposeRule
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.test.core.app.ApplicationProvider
import com.mmoragues.herois.MainActivity
import com.mmoragues.herois.R.string.heroesFavorites_empty
import com.mmoragues.herois.R.string.heroesList_noResult
import com.mmoragues.herois.common.TestConstants
import com.mmoragues.herois.di.AppModule
import com.mmoragues.herois.ui.navigation.BottomMenuOption
import com.mmoragues.herois.ui.theme.HeroisTheme
import dagger.hilt.android.testing.HiltAndroidRule
import dagger.hilt.android.testing.HiltAndroidTest
import dagger.hilt.android.testing.UninstallModules
import org.junit.Before
import org.junit.Rule
import org.junit.Test

//Proves de la pantalla Favorites
@HiltAndroidTest
@UninstallModules(AppModule::class)
class FavoritesScreenTest{
    @get:Rule(order = 0)
    val hiltRule = HiltAndroidRule(this)

    @get:Rule(order = 1)
    val composeRule = createAndroidComposeRule<MainActivity>()

    @Before
    fun setUp(){
        hiltRule.inject()
        composeRule.setContent {
            val navController = rememberNavController()
            HeroisTheme {
                NavHost(
                    navController = navController,
                    startDestination = BottomMenuOption.Favorite.route
                ){
                    composable(route = BottomMenuOption.Favorite.route){
                        FavoritesScreen(navController = navController)
                    }
                }
            }
        }
    }

    /**
     * Comprovaem que surt el heroi guardat com a favorit i que el podem desseleccionar i ja no surt a la llista
     * Es comprova que surt el missatge que ja no hi ha herois a la llista
     */
    @Test
    fun favoriteScreen_UncheckAndFilter(){
        val context = ApplicationProvider.getApplicationContext<Context>()
        val heroeFavoritesEmpty = context.getString(heroesFavorites_empty)
        val noResults = context.getString(heroesList_noResult)

        //Comprovem que al fer una búsqueda erronia surt el missatge corresponent
        composeRule.onNodeWithTag("Card a").assertExists()
        composeRule.onNodeWithContentDescription("Search Icon").performClick()
        composeRule.onNodeWithTag(TestConstants.SEARCHTEXT).performTextInput("inventat")
        composeRule.onNodeWithText(noResults, true).assertExists()

        //Comprovem que al no haver-hi cap heroi a la llista de favorits surt el missatge corresponent
        composeRule.onNodeWithContentDescription("Delete Icon").performClick()
        composeRule.onNodeWithTag("Card a").assertExists()
        composeRule.onNodeWithTag("Card a").onChildren().filterToOne(hasContentDescription("Favorite Icon")).performClick()
        composeRule.onNodeWithTag("Card a").assertDoesNotExist()
        composeRule.onNodeWithText(heroeFavoritesEmpty, true).assertExists()

    }
}