package com.mmoragues.herois.ui.screens.gameScreen

import android.content.Context
import androidx.compose.ui.test.*
import androidx.compose.ui.test.junit4.createAndroidComposeRule
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.test.core.app.ApplicationProvider
import com.mmoragues.herois.MainActivity
import com.mmoragues.herois.R
import com.mmoragues.herois.common.TestConstants.BETBOTTOMS
import com.mmoragues.herois.common.TestConstants.BOTTOMCARD
import com.mmoragues.herois.common.TestConstants.TOPCARD
import com.mmoragues.herois.di.AppModule
import com.mmoragues.herois.ui.navigation.BottomMenuOption
import com.mmoragues.herois.ui.theme.HeroisTheme
import dagger.hilt.android.testing.HiltAndroidRule
import dagger.hilt.android.testing.HiltAndroidTest
import dagger.hilt.android.testing.UninstallModules
import org.junit.Before
import org.junit.Rule
import org.junit.Test

//Prova de la pantalla del joc
@HiltAndroidTest
@UninstallModules(AppModule::class)
class GameScreenTest {
    @get:Rule(order = 0)
    val hiltRule = HiltAndroidRule(this)

    @get:Rule(order = 1)
    val composeRule = createAndroidComposeRule<MainActivity>()

    @Before
    fun setUp() {
        hiltRule.inject()
        composeRule.setContent {
            val navController = rememberNavController()
            HeroisTheme {
                NavHost(
                    navController = navController,
                    startDestination = BottomMenuOption.Game.route
                ) {
                    composable(route = BottomMenuOption.Game.route) {
                        GameScreen(navController = navController)
                    }
                }
            }
        }
    }

    //Comprovem que la pantalla del joc funciona correctament
    @Test
    fun game_Works(){
        val context = ApplicationProvider.getApplicationContext<Context>()
        val combat: String = context.getString(R.string.power_combat)
        val result: String = context.getString(R.string.result_desc_win)

        //Confirmem que les targes existeixen
        composeRule.onNodeWithTag(TOPCARD).assertExists()
        composeRule.onNodeWithTag(BOTTOMCARD).assertExists()

        //Jugem amb la carta superior i confirme que surten els botons per apostar
        composeRule.onNodeWithTag(TOPCARD).performClick()
        composeRule.onAllNodesWithTag(BETBOTTOMS, true).assertCountEquals(2)

        //Apostem a un superpoder i que desaparexen els botoms
        composeRule.onNodeWithText(combat).performClick()
        composeRule.onAllNodesWithTag(BETBOTTOMS, true).assertCountEquals(0)

        //Confirmem que surt la barra amb el poder correcte tant a la carta superior com a la inferior
        composeRule.onNode(hasContentDescription("BackCard") and hasTestTag(TOPCARD)).assertExists()
        composeRule.onNode(hasContentDescription("BackCard") and hasTestTag(TOPCARD)).assertTextContains(combat)
        composeRule.onNode(hasContentDescription("BackCard") and hasTestTag(BOTTOMCARD)).assertTextContains(combat)
    }

}