package com.mmoragues.herois.ui.screens.randomScreen

import androidx.compose.ui.test.*
import androidx.compose.ui.test.junit4.createAndroidComposeRule
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.mmoragues.herois.MainActivity
import com.mmoragues.herois.common.TestConstants.APPEARANCESECTION
import com.mmoragues.herois.common.TestConstants.APPEARANCESECTIONDETAIL
import com.mmoragues.herois.common.TestConstants.BIOGRAPHYSECTION
import com.mmoragues.herois.common.TestConstants.BIOGRAPHYSECTIONDETAIL
import com.mmoragues.herois.common.TestConstants.CONNECTIONSECTION
import com.mmoragues.herois.common.TestConstants.CONNECTIONSECTIONDETAIL
import com.mmoragues.herois.common.TestConstants.IMAGESECTION
import com.mmoragues.herois.common.TestConstants.POWERSECTION
import com.mmoragues.herois.common.TestConstants.WORKSECTION
import com.mmoragues.herois.common.TestConstants.WORKSECTIONDETAIL
import com.mmoragues.herois.di.AppModule
import com.mmoragues.herois.ui.navigation.BottomMenuOption
import com.mmoragues.herois.ui.theme.HeroisTheme
import dagger.hilt.android.testing.HiltAndroidRule
import dagger.hilt.android.testing.HiltAndroidTest
import dagger.hilt.android.testing.UninstallModules
import org.junit.Before
import org.junit.Rule
import org.junit.Test

// Proves de la pantalla que mostra un heroi a l'atzar
@HiltAndroidTest
@UninstallModules(AppModule::class)
class RandomScreenTest {

    @get:Rule(order = 0)
    val hiltRule = HiltAndroidRule(this)

    @get:Rule(order = 1)
    val composeRule = createAndroidComposeRule<MainActivity>()

    @Before
    fun setUp() {
        hiltRule.inject()
        composeRule.setContent {
            val navController = rememberNavController()
            HeroisTheme {
                NavHost(
                    navController = navController,
                    startDestination = BottomMenuOption.Random.route
                ) {
                    composable(route = BottomMenuOption.Random.route) {
                        RandomScreen(navController = navController)
                    }
                }
            }
        }
    }

    /**
     * Comprovem que es mostra en heroi aleatori i que es pot tornar a demanar nou heroi
     */
    @Test
    fun randomHeroIcon_ShowNewRandomHero() {

        //Validem que surt el heroi aleatori inicial
        composeRule.onNodeWithContentDescription("Random Icon").assertIsDisplayed()
        composeRule.onNodeWithTag(IMAGESECTION).assertExists()
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(POWERSECTION))
        composeRule.onNodeWithTag(POWERSECTION).assertExists()
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(BIOGRAPHYSECTION))
        composeRule.onNodeWithTag(BIOGRAPHYSECTION).assertExists()
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(APPEARANCESECTION))
        composeRule.onNodeWithTag(APPEARANCESECTION).assertExists()
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(CONNECTIONSECTION))
        composeRule.onNodeWithTag(CONNECTIONSECTION).assertExists()
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(WORKSECTION))
        composeRule.onNodeWithTag(WORKSECTION).assertExists()

        //Validem que es pot demanar un nou heroi aleatori
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasContentDescription("Random Icon"))
        composeRule.onNodeWithContentDescription("Random Icon").performClick()
        composeRule.onNodeWithContentDescription("Random Icon").assertIsDisplayed()
        composeRule.onNodeWithTag(IMAGESECTION).assertExists()
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(POWERSECTION))
        composeRule.onNodeWithTag(POWERSECTION).assertExists()
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(BIOGRAPHYSECTION))
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(APPEARANCESECTION))
        composeRule.onNodeWithTag(APPEARANCESECTION).assertExists()
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(CONNECTIONSECTION))
        composeRule.onNodeWithTag(CONNECTIONSECTION).assertExists()
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(WORKSECTION))
        composeRule.onNodeWithTag(WORKSECTION).assertExists()
    }

    /**
     * Comprovem que estan totes les seccions i es poden obrir i tancar correctament
     */
    @Test
    fun expandabledSections_ShowInfo(){
        //Es movem al final de la pantalla
        composeRule.onNodeWithTag("LazyColumn").performScrollToNode(hasTestTag(WORKSECTION))

        //Comprovem que la seccion de Biografia es mostra i s'amaga corrrectament
        composeRule.onNodeWithTag(BIOGRAPHYSECTION).performClick()
        composeRule.onNodeWithTag(BIOGRAPHYSECTIONDETAIL, useUnmergedTree = true).assertExists()
        composeRule.onNodeWithTag(BIOGRAPHYSECTION).performClick()
        composeRule.onNodeWithTag(BIOGRAPHYSECTIONDETAIL, useUnmergedTree = true).assertDoesNotExist()

        //Comprovem que la seccion d'Aparença es mostra i s'amaga corrrectament
        composeRule.onNodeWithTag(APPEARANCESECTION).performClick()
        composeRule.onNodeWithTag(APPEARANCESECTIONDETAIL, useUnmergedTree = true).assertExists()
        composeRule.onNodeWithTag(APPEARANCESECTION).performClick()
        composeRule.onNodeWithTag(APPEARANCESECTIONDETAIL, useUnmergedTree = true).assertDoesNotExist()

        //Comprovem que la seccion de Connexions es mostra i s'amaga corrrectament
        composeRule.onNodeWithTag(CONNECTIONSECTION).performClick()
        composeRule.onNodeWithTag(CONNECTIONSECTIONDETAIL, useUnmergedTree = true).assertExists()
        composeRule.onNodeWithTag(CONNECTIONSECTION).performClick()
        composeRule.onNodeWithTag(CONNECTIONSECTIONDETAIL, useUnmergedTree = true).assertDoesNotExist()

        //Comprovem que la seccion de Treball es mostra i s'amaga corrrectament
        composeRule.onNodeWithTag(WORKSECTION).performClick()
        composeRule.onNodeWithTag(WORKSECTIONDETAIL, useUnmergedTree = true).assertExists()
        composeRule.onNodeWithTag(WORKSECTION).performClick()
        composeRule.onNodeWithTag(WORKSECTIONDETAIL, useUnmergedTree = true).assertDoesNotExist()

    }

}