package com.mmoragues.herois.data

import com.mmoragues.herois.data.database.dto.*
import com.mmoragues.herois.data.remote.dto.*
import com.mmoragues.herois.domain.model.*

/**
 * Funcions per mapejar del classes de la BBDD al Model del Domini
 */
fun DbAppearance.toAppearance(): Appearance =
    Appearance(
        eyeColor = eyeColor,
        gender = gender,
        hairColor = hairColor,
        height = height,
        race = race,
        weight = weight
    )

fun DbBiography.toBiography(): Biography =
    Biography(
        aliases = aliases,
        alignment = alignment,
        alterEgos = alterEgos,
        firstAppearance = firstAppearance,
        fullName = fullName,
        placeOfBirth = placeOfBirth,
        publisher = publisher
    )

fun DbConnections.toConnections(): Connections =
    Connections(
        groupAffiliation = groupAffiliation,
        relatives = relatives
    )

fun DbImages.toImages(): Images =
    Images(
        lg = lg,
        md = md,
        sm = sm,
        xs = xs
    )

fun DbPowerstats.toPowerstats(): Powerstats =
    Powerstats(
        combat = combat,
        durability = durability,
        intelligence = intelligence,
        power = power,
        speed = speed,
        strength = strength
    )

fun DbWork.toWork(): Work =
    Work(
        base,
        occupation
    )

fun DbHero.toHero(): Hero =
    Hero(
        id = id,
        name = name,
        isFavorite = isFavorite,
        appearance = appearance.toAppearance(),
        biography = biography.toBiography(),
        connections = connections.toConnections(),
        images = images.toImages(),
        powerstats = powerstats.toPowerstats(),
        work = work.toWork()
    )

/**
 * Funcions per mapejar les classes del servidor Remot a les del Model del Domini
 */

fun RemoteAppearance.toAppearance(): Appearance =
    Appearance(
        eyeColor = eyeColor ?: "-",
        gender = gender ?: "-",
        hairColor = hairColor ?: "-",
        height = height ?: emptyList(),
        race = race ?: "-",
        weight = weight ?: emptyList()
    )

fun RemoteBiography.toBiography(): Biography =
    Biography(
        aliases = aliases ?: emptyList(),
        alignment = alignment ?: "-",
        alterEgos = alterEgos ?: "-",
        firstAppearance = firstAppearance ?: "-",
        fullName = fullName ?: "-",
        placeOfBirth = placeOfBirth ?: "-",
        publisher = publisher ?: "-"
    )

fun RemoteConnections.toConnections(): Connections =
    Connections(
        groupAffiliation = groupAffiliation ?: "-",
        relatives = relatives ?: "-"
    )

fun RemoteImages.toImages(): Images =
    Images(
        lg = lg ?: "-",
        md = md ?: "-",
        sm = sm ?: "-",
        xs = xs ?: "-"
    )

fun RemotePowerstats.toPowerstats(): Powerstats =
    Powerstats(
        combat = if (combat ?: 0 > 100) 100 else combat ?: 0,
        durability = if (durability ?: 0 > 100) 100 else durability ?: 0,
        intelligence = if (intelligence ?: 0 > 100) 100 else intelligence ?: 0,
        power = if (power ?: 0 > 100) 100 else power ?: 0,
        speed = if (speed ?: 0 > 100) 100 else speed ?: 0,
        strength = if (strength ?: 0 > 100) 100 else strength ?: 0
    )

fun RemoteWork.toWork(): Work =
    Work(
        base ?: "-",
        occupation ?: "-"
    )

fun RemoteHero.toHero(): Hero =
    Hero(
        id = id ?: 0,
        name = name ?: "-",
        appearance = appearance!!.toAppearance(),
        biography = biography!!.toBiography(),
        connections = connections!!.toConnections(),
        images = images!!.toImages(),
        powerstats = powerstats!!.toPowerstats(),
        work = work!!.toWork()
    )

/**
 * Funcions per mapejar del Model del Domini a les Classes de la BBDD
 */

fun Appearance.toDbAppearance(): DbAppearance =
    DbAppearance(
        eyeColor = eyeColor,
        gender = gender,
        hairColor = hairColor,
        height = height,
        race = race,
        weight = weight
    )

fun Biography.toDbBiography(): DbBiography =
    DbBiography(
        aliases = aliases,
        alignment = alignment,
        alterEgos = alterEgos,
        firstAppearance = firstAppearance,
        fullName = fullName,
        placeOfBirth = placeOfBirth,
        publisher = publisher
    )

fun Connections.toDbConnections(): DbConnections =
    DbConnections(
        groupAffiliation = groupAffiliation,
        relatives = relatives
    )

fun Images.toDbImages(): DbImages =
    DbImages(
        lg = lg,
        md = md,
        sm = sm,
        xs = xs
    )

fun Powerstats.toDbPowerstats(): DbPowerstats =
    DbPowerstats(
        combat = combat,
        durability = durability,
        intelligence = intelligence,
        power = power,
        speed = speed,
        strength = strength
    )

fun Work.toDbWork(): DbWork =
    DbWork(
        base = base,
        occupation = occupation
    )

fun Hero.toDbHero(): DbHero =
    DbHero(
        id = id,
        name = name,
        isFavorite = isFavorite,
        appearance = appearance.toDbAppearance(),
        biography = biography.toDbBiography(),
        connections = connections.toDbConnections(),
        images = images.toDbImages(),
        powerstats = powerstats.toDbPowerstats(),
        work = work.toDbWork()
    )