package com.mmoragues.herois.data.database

import com.mmoragues.herois.data.toDbHero
import com.mmoragues.herois.data.toHero
import com.mmoragues.herois.domain.model.Hero
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.flow.map

/**
 * Implementació de la interficie LocalDatasource
 * En el nostre cas, la implementem fent servir una BBDD Room
 */
class LocalDataSourceImpl(db: HeroDatabase) : LocalDataSource {
    private val heroDao = db.heroDao

    override suspend fun insertHeroes(heroes: List<Hero>) {
        heroes.forEach { hero ->
            val oldHero = heroDao.getHeroById(hero.id).first()
            if (oldHero != null) {
                hero.isFavorite = oldHero.isFavorite
            }
            heroDao.insertHero(hero.toDbHero())
        }
    }

    override fun getAllHeroes(): Flow<List<Hero>> {
        return heroDao.getAllHeroes().map { heroes -> heroes.map { it.toHero() } }
    }

    override fun getHeroById(id: Int): Flow<Hero> {
        return heroDao.getHeroById(id).map { it.toHero() }
    }

    override suspend fun toggleIsFavorite(id: Int) {
        heroDao.toggleIsFavorite(id)
    }

    override fun getFavoritesHeroes(): Flow<List<Hero>> {
        return heroDao.getFavoritesHeroes().map { heroes -> heroes.map { it.toHero() } }
    }

    override suspend fun getRandomHero(): Hero {
        return heroDao.getRandomHero().toHero()
    }
}