package com.mmoragues.herois.data.repository

import com.mmoragues.herois.common.networkBoundResource
import com.mmoragues.herois.data.database.LocalDataSource
import com.mmoragues.herois.data.remote.RemoteDataSource
import com.mmoragues.herois.domain.model.Hero
import com.mmoragues.herois.domain.repository.HeroRepository
import kotlinx.coroutines.flow.Flow

/**
 * Implementació del repository
 * Com a paràmetres se'ls hi passa les interficies de la LocalDataSource i la RemoteDatasource, no les seves implementacions
 * D'aquesta manera desacoplem el codi i facilita futurs canvis
 */
class HeroRepositoryImpl(
    private val localDataSource: LocalDataSource,
    private val remoteDataSource: RemoteDataSource
):HeroRepository {

    override fun getAllHeroes() = networkBoundResource(
        query = {
            localDataSource.getAllHeroes()
        },
        fetch = {
            remoteDataSource.getAllHeroes()
        },
        saveFetchResult = {
            localDataSource.insertHeroes(it)
        }
    )

    override fun getHeroById(id: Int): Flow<Hero> {
        return localDataSource.getHeroById(id)
    }

    override suspend fun toggleIsFavorite(id: Int) {
        localDataSource.toggleIsFavorite(id)
    }

    override suspend fun getRandomHero(): Hero {
        return localDataSource.getRandomHero()
    }

    override fun getFavoritesHeroes(): Flow<List<Hero>> {
        return localDataSource.getFavoritesHeroes()
    }
}