package com.mmoragues.herois.ui.navigation

import androidx.compose.runtime.Composable

import androidx.navigation.NavHostController
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.navArgument
import com.mmoragues.herois.ui.screens.favoritesScreen.FavoritesScreen
import com.mmoragues.herois.ui.screens.gameScreen.GameScreen
import com.mmoragues.herois.ui.screens.heroDetailScreen.HeroDetailScreen
import com.mmoragues.herois.ui.screens.homeScreen.HomeScreen
import com.mmoragues.herois.ui.screens.randomScreen.RandomScreen
import com.mmoragues.herois.ui.screens.splashScreen.SplashScreen

/**
 * Funció que controla la navegació entre les diferents pantalles
 */
@Composable
fun Navigation(navController: NavHostController) {

    NavHost(
        navController = navController,
        startDestination = Screen.SplashScreen.route
    ) {

        //Pantalla Splash
        composable(
            route = Screen.SplashScreen.route
        ) {
            SplashScreen(navController = navController)
        }

        //Pantalla Home
        composable(
            route = BottomMenuOption.Home.route
        ) {
            HomeScreen(navController = navController)
        }

        //Pantalla Favorits
        composable(
            route = BottomMenuOption.Favorite.route
        ) {
            FavoritesScreen(navController = navController)

        }

        //Pantalla del Joc
        composable(BottomMenuOption.Game.route) {
            GameScreen(navController = navController)

        }

        //Pantalla amb el detall de l'heroi
        composable(
            route = Screen.HeroDetailScreen.route + "/{heroId}",
            arguments = listOf(
                navArgument("heroId") {
                    type = NavType.IntType
                    defaultValue = 0
                    nullable = false
                }
            )
        ) { entry ->
            HeroDetailScreen(navController = navController)
        }

        //Pantalla de l'heroi aleatori
        composable(
            route = BottomMenuOption.Random.route
        ) {
            RandomScreen(navController = navController)
        }
    }
}