package com.mmoragues.herois.ui.screens.favoritesScreen

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavHostController
import com.mmoragues.herois.R
import com.mmoragues.herois.ui.navigation.Screen
import com.mmoragues.herois.ui.sharedComponents.BottomBar
import com.mmoragues.herois.ui.sharedComponents.ItemHeroCard
import com.mmoragues.herois.ui.sharedComponents.MainScaffold
import com.mmoragues.herois.ui.sharedComponents.searchBar.MainAppBar
import com.mmoragues.herois.ui.sharedComponents.searchBar.SearchWidgetState

/**
 * Pantalla on es motra el llistat d'herois favorits
 */
@Composable
fun FavoritesScreen(
    navController: NavHostController,
    viewModel: FavoritesViewModel = hiltViewModel()
) {

    val favoritesHeroes = viewModel.favoritesHeroes
    val searchWidgetState = viewModel.searchWidgetState
    val searchTextState = viewModel.searchTextState
    val noFavoritesHeroesMessage = viewModel.noFavoritesHeroesMessage

    MainScaffold(
        topBar = {
            MainAppBar(
                title = stringResource(id = R.string.favoritesScreen_title),
                searchWidgetState = searchWidgetState,
                searchTextState = searchTextState,
                onTextChange = {
                    viewModel.onEvent(FavoritesScreenEvent.OnUpdateSearchTextState(it))
                },
                onCloseClicked = {
                    viewModel.onEvent(
                        FavoritesScreenEvent.OnUpdateSearchWidgetState(
                            SearchWidgetState.CLOSED
                        )
                    )
                },
                onSearchClicked = {},
                onSearchTriggered = {
                    viewModel.onEvent(
                        FavoritesScreenEvent.OnUpdateSearchWidgetState(
                            SearchWidgetState.OPENED
                        )
                    )
                }
            )
        },
        bottomBar = { BottomBar(navController = navController) }
    )
    { paddingValues ->

        //Si no hi ha herois a la llista de favorits, es mostra el missatge indicant el motiu
        // Pot ser perquè la búsqueda no té resultats o perquè l'usuari no ha guardat cap heroi com a favorit
        if (noFavoritesHeroesMessage != -1) {
            Column(
                modifier = Modifier.fillMaxSize(),
                verticalArrangement = Arrangement.Center,
                horizontalAlignment = Alignment.CenterHorizontally
            ) {
                Text(
                    text = stringResource(id = noFavoritesHeroesMessage),
                    color = MaterialTheme.colors.error,
                    style = MaterialTheme.typography.h6
                )
            }
        }

        //LListat amb els herois favorits
        LazyColumn(
            modifier = Modifier.fillMaxSize(),
            contentPadding = paddingValues
        ) {
            items(favoritesHeroes) { hero ->
                ItemHeroCard(
                    hero = hero,
                    onItemClicked = { hero ->
                        navController.navigate(Screen.HeroDetailScreen.route + "/${hero.id}")
                    },
                    onFavoriteClicked = { hero ->
                        viewModel.onEvent(
                            FavoritesScreenEvent.OnFavoriteClick(hero)
                        )
                    }
                )
            }
        }
    }

}