package com.mmoragues.herois.ui.screens.gameScreen


import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.Icon
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.material.TopAppBar
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.testTag
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavHostController
import com.mmoragues.herois.R
import com.mmoragues.herois.common.TestConstants.BOTTOMCARD
import com.mmoragues.herois.common.TestConstants.TOPCARD
import com.mmoragues.herois.ui.screens.gameScreen.flipCard.BackCard
import com.mmoragues.herois.ui.screens.gameScreen.flipCard.CardPosition
import com.mmoragues.herois.ui.screens.gameScreen.flipCard.FlipCard
import com.mmoragues.herois.ui.screens.gameScreen.flipCard.FrontCard
import com.mmoragues.herois.ui.sharedComponents.BottomBar
import com.mmoragues.herois.ui.sharedComponents.MainScaffold

/**
 * Pantall del Joc
 */
@Composable
fun GameScreen(
    navController: NavHostController,
    viewModel: GameViewModel = hiltViewModel()
) {

    //Variable d'estat de la Game Screen
    val heroTopCard = viewModel.heroTopCard
    val topCardIsMyCard = viewModel.topCardIsMyCard
    val heroBottomCard = viewModel.heroBottomCard
    val bottomCardIsMyCard = viewModel.bottomCardIsMyCard
    val topCardFace = viewModel.topCardFace
    val bottomCardFace = viewModel.bottomCardFace
    val titleId = viewModel.titleId
    val startBetAnimation = viewModel.startBetAnimation
    val chosenPower = viewModel.chosenPower
    val chosenPowerValueTopCard = viewModel.chosenPowerValueTopCard
    val chosenPowerValueBottomCard = viewModel.chosenPowerValueBottomCard
    val youHaveWon = viewModel.youHaveWon
    val youHaveLost = viewModel.youHaveLost
    val youHaveTied = viewModel.youHaveTied

    MainScaffold(
        topBar = {
            TopAppBar(
                title = { Text(text = stringResource(id = titleId)) },
                backgroundColor = MaterialTheme.colors.background,
                contentColor = Color.White,
                navigationIcon = {
                    Icon(
                        imageVector = Icons.Default.ArrowBack,
                        contentDescription = null,
                        modifier = Modifier
                            .size(24.dp, 24.dp)
                            .clickable {
                                navController.navigateUp()
                            },
                        tint = Color.White
                    )
                }
            )
        },
        bottomBar = { BottomBar(navController = navController) },
    ) { paddingValues ->
        Column(
            modifier = Modifier
                .fillMaxSize()
                .verticalScroll(rememberScrollState())
                .background(MaterialTheme.colors.background)
                .padding(paddingValues),
            verticalArrangement = Arrangement.SpaceEvenly
        ) {
            //Carta superior
            FlipCard(
                modifier = Modifier.testTag(TOPCARD),
                cardFace = topCardFace,
                onClick = { viewModel.onEvent(GameScreenEvent.OnTopCardClick)},
                front = {
                    FrontCard(
                        imageId = R.drawable.front_card,
                        modifier = Modifier.fillMaxSize()
                    )
                },
                back = {
                    BackCard(
                        navController = navController,
                        hero = heroTopCard,
                        position = CardPosition.TOP,
                        onEvent = { viewModel.onEvent(it) },
                        isMyCard = topCardIsMyCard,
                        startBetAnimation = startBetAnimation,
                        chosenPower = chosenPower,
                        chosenPowerValue = chosenPowerValueTopCard,
                        youHaveWon = youHaveWon,
                        youHaveLost = youHaveLost,
                        youHaveTied = youHaveTied
                    )
                }
            )

            //Carta inferior
            FlipCard(
                modifier = Modifier.testTag(BOTTOMCARD),
                cardFace = bottomCardFace,
                onClick = { viewModel.onEvent(GameScreenEvent.OnBottomCardClick) },
                front = {
                    FrontCard(
                        imageId = R.drawable.front_card,
                        modifier = Modifier.fillMaxSize()
                    )
                },
                back = {
                    BackCard(
                        navController = navController,
                        hero = heroBottomCard,
                        position = CardPosition.BOTTOM,
                        onEvent = { viewModel.onEvent(it) },
                        isMyCard = bottomCardIsMyCard,
                        startBetAnimation = startBetAnimation,
                        chosenPower = chosenPower,
                        chosenPowerValue = chosenPowerValueBottomCard,
                        youHaveWon = youHaveWon,
                        youHaveLost = youHaveLost,
                        youHaveTied = youHaveTied
                    )
                }
            )

        }
    }

}