package com.mmoragues.herois.ui.screens.gameScreen.flipCard

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.Button
import androidx.compose.material.ButtonDefaults
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.compose.ui.window.Dialog
import androidx.navigation.NavController
import com.airbnb.lottie.compose.LottieAnimation
import com.airbnb.lottie.compose.LottieCompositionSpec
import com.airbnb.lottie.compose.animateLottieCompositionAsState
import com.airbnb.lottie.compose.rememberLottieComposition
import com.mmoragues.herois.R
import com.mmoragues.herois.ui.screens.gameScreen.GameScreenEvent
import com.mmoragues.herois.ui.theme.Shapes
import com.mmoragues.herois.ui.theme.heroGreenColor
import com.mmoragues.herois.ui.theme.heroRedColor

/**
 * Composable que mostra el diàleg amb el resultat final i els botons per torna a jugar o sortir
 */
@Composable
fun ResultDialog(
    navController: NavController,
    onEvent: (GameScreenEvent) -> Unit,
    title: String,
    desc: String,
    rawRes: Int,
    color: Color
) {
    Dialog(onDismissRequest = {
        onEvent(GameScreenEvent.OnExitClick)
        navController.navigateUp()
    }) {
        Box(
            modifier = Modifier
                .width(300.dp)
                .height(400.dp)
        ) {
            Column(
                modifier = Modifier
                    .size(300.dp)
            ) {
                Spacer(modifier = Modifier.height(36.dp))
                Box(
                    modifier = Modifier
                        .width(300.dp)
                        .height(164.dp)
                        .background(
                            color = Color.White,
                            shape = RoundedCornerShape(10.dp)
                        )
                ) {
                    Column(
                        modifier = Modifier.padding(12.dp),
                        horizontalAlignment = Alignment.CenterHorizontally
                    ) {
                        Spacer(modifier = Modifier.height(24.dp))
                        Text(
                            text = title.uppercase(),
                            style = TextStyle(
                                color = Color.Black,
                                fontSize = 18.sp,
                                fontWeight = FontWeight.Bold
                            )
                        )
                        Spacer(modifier = Modifier.height(8.dp))
                        Text(
                            text = desc,
                            style = TextStyle(
                                color = Color.Black,
                                fontSize = 14.sp,
                                fontWeight = FontWeight.Normal
                            )
                        )
                        Spacer(modifier = Modifier.height(24.dp))
                        Row(
                            modifier = Modifier
                                .fillMaxWidth()
                                .fillMaxHeight()
                        ) {
                            Button(
                                onClick = {
                                    onEvent(GameScreenEvent.OnExitClick)
                                    navController.navigateUp()
                                },
                                shape = Shapes.large,
                                colors = ButtonDefaults.buttonColors(backgroundColor = heroRedColor),
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .weight(1f)
                                    .clip(RoundedCornerShape(5.dp)),
                                contentPadding = PaddingValues(0.dp)
                            ) {
                                Text(
                                    text = stringResource(id = R.string.card_exit),
                                    color = Color.White,
                                )
                            }

                            Spacer(modifier = Modifier.width(8.dp))

                            Button(
                                onClick = { onEvent(GameScreenEvent.OnRestartClick) },
                                shape = Shapes.large,
                                colors = ButtonDefaults.buttonColors(backgroundColor = heroGreenColor),
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .weight(1f)
                                    .clip(RoundedCornerShape(5.dp)),
                                contentPadding = PaddingValues(0.dp)
                            ) {
                                Text(
                                    text = stringResource(id = R.string.card_play_again),
                                    color = Color.White,
                                )
                            }
                        }
                    }
                }
            }
            DialogHeader(
                modifier = Modifier
                    .size(75.dp)
                    .align(Alignment.TopCenter)
                    .background(color, CircleShape)
                    .border(
                        border = BorderStroke(width = 5.dp, color = Color.White),
                        shape = CircleShape
                    ), rawRes = rawRes
            )
        }

    }

}

//Capçalera amb l'Animació de guanyar, perdre o empatar
@Composable
fun DialogHeader(modifier: Modifier, rawRes: Int) {
    val composition by rememberLottieComposition(spec = LottieCompositionSpec.RawRes(rawRes))
    val progress by animateLottieCompositionAsState(composition = composition)

    LottieAnimation(
        composition = composition,
        progress = progress,
        modifier = modifier,
        contentScale = ContentScale.Crop
    )

}