package com.mmoragues.herois.ui.screens.heroDetailScreen

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.size
import androidx.compose.material.*

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavHostController
import com.mmoragues.herois.ui.sharedComponents.heroDetails.HeroDetails

/**
 * Pantall amb el detall dels herois
 */
@Composable
fun HeroDetailScreen(
    navController: NavHostController,
    viewModel: HeroDetailViewModel = hiltViewModel()
) {

    val hero = viewModel.hero

    Scaffold(
        topBar = {
            TopAppBar(
                title = { Text(text = hero.name) },
                backgroundColor = MaterialTheme.colors.background,
                contentColor = Color.White,
                navigationIcon = {
                    Icon(
                        imageVector = Icons.Default.ArrowBack,
                        contentDescription = "Arrow Back",
                        modifier = Modifier
                            .size(24.dp, 24.dp)
                            .clickable {
                                navController.navigateUp()
                            },
                        tint = Color.White
                    )
                }
            )
        }
    ) { paddingValues ->

        HeroDetails(
            hero = hero,
            paddingValues = paddingValues,
            onFavoriteClicked = { hero ->
                viewModel.onEvent(
                    HeroDetailEvent.OnFavoriteClick(hero)
                )
            }
        )
    }

}