package com.mmoragues.herois.ui.screens.heroDetailScreen

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.mmoragues.herois.domain.model.Hero
import com.mmoragues.herois.domain.useCase.GetHeroByIdUseCase
import com.mmoragues.herois.domain.useCase.ToggleFavoriteUseCase
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.flow.last
import kotlinx.coroutines.flow.launchIn
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * View Model de la pantalla de Detalls dels Herois
 */
@HiltViewModel
class HeroDetailViewModel @Inject constructor(
    private val getHeroByIdUseCase: GetHeroByIdUseCase,
    private val toggleFavoriteUseCase: ToggleFavoriteUseCase,
    savedStateHandle: SavedStateHandle
) : ViewModel() {

    var hero by mutableStateOf(Hero())
        private set

    init {
        savedStateHandle.get<Int>("heroId")?.let { heroId ->
            getHero(heroId)
        }
    }

    private fun getHero(heroId: Int) {

        getHeroByIdUseCase(heroId).onEach { result ->
            hero = result
        }.launchIn(viewModelScope)
    }

    fun onEvent(event: HeroDetailEvent) {
        when (event) {
            is HeroDetailEvent.OnFavoriteClick -> {
                viewModelScope.launch {
                    toggleFavoriteUseCase(event.hero)
                }
            }
        }
    }
}