package com.mmoragues.herois.ui.screens.randomScreen

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.size
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavHostController
import com.mmoragues.herois.ui.sharedComponents.BottomBar
import com.mmoragues.herois.ui.sharedComponents.heroDetails.HeroDetails

/**
 * Pantalla on es mostra un heroi aleatori
 */
@Composable
fun RandomScreen(
    navController: NavHostController,
    viewModel: RandomViewModel = hiltViewModel()
) {
    val randomHero = viewModel.randomHero

    Scaffold(
        topBar = {
            TopAppBar(
                title = { Text(text = randomHero.name) },
                backgroundColor = MaterialTheme.colors.background,
                contentColor = Color.White,
                navigationIcon = {
                    Icon(
                        imageVector = Icons.Default.ArrowBack,
                        contentDescription = null,
                        modifier = Modifier
                            .size(24.dp, 24.dp)
                            .clickable {
                                navController.navigateUp()
                            },
                        tint = Color.White
                    )
                }
            )
        },
        bottomBar = { BottomBar(navController = navController) }
    ) { paddingValues ->

        HeroDetails(
            hero = randomHero,
            isRandom = true,
            paddingValues = paddingValues,
            onFavoriteClicked = { hero ->
                viewModel.onEvent(RandomScreenEvent.OnFavoriteClick(hero))
            },
            onRandomClicked = { viewModel.onEvent(RandomScreenEvent.OnRandomClick)}
        )
    }

}