package com.mmoragues.herois.ui.screens.randomScreen

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.mmoragues.herois.domain.model.Hero
import com.mmoragues.herois.domain.useCase.GetHeroByIdUseCase
import com.mmoragues.herois.domain.useCase.GetRandomHeroUseCase
import com.mmoragues.herois.domain.useCase.ToggleFavoriteUseCase
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.flow.launchIn
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * View Model de la pantalla d'Horoi aleatori
 */
@HiltViewModel
class RandomViewModel @Inject constructor(
    private val getRandomHeroUseCase: GetRandomHeroUseCase,
    private val toggleFavoriteUseCase: ToggleFavoriteUseCase,
    private val getHeroByIdUseCase: GetHeroByIdUseCase
) : ViewModel() {

    var randomHero by mutableStateOf(Hero())
        private set

    init {
        getRandomHero()
    }

    private fun getRandomHero(){
        viewModelScope.launch {
            randomHero = getRandomHeroUseCase()
        }

    }

    fun onEvent(event: RandomScreenEvent){
        when(event){
            is RandomScreenEvent.OnFavoriteClick ->{
                viewModelScope.launch {
                    toggleFavoriteUseCase(event.hero)
                    randomHero =getHeroByIdUseCase(event.hero.id).first()
                }
            }
            is RandomScreenEvent.OnRandomClick -> {
                getRandomHero()
            }
        }
    }

}