package com.mmoragues.herois.ui.screens.splashScreen

import androidx.compose.animation.core.animateFloatAsState
import androidx.compose.animation.core.tween
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.alpha
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.navigation.NavHostController
import com.mmoragues.herois.R
import com.mmoragues.herois.ui.navigation.BottomMenuOption
import kotlinx.coroutines.delay

/**
 * Pantalla Splash que es mostra durant 1.5 segons.
 * A continuació es navega cap a la pàgina d'inici
 */
@Composable
fun SplashScreen(navController: NavHostController){

    var startAnimation by remember { mutableStateOf(false) }
    val alphaAnim = animateFloatAsState(
        targetValue = if (startAnimation) 1f else 0f,
        animationSpec = tween(
            durationMillis = 1500
        )
    )

    LaunchedEffect(key1 = true) {
        startAnimation = true
        delay(1500)
        navController.popBackStack()
        navController.navigate(BottomMenuOption.Home.route)
    }
    Splash(alpha = alphaAnim.value)

}

@Composable
fun Splash(alpha: Float){
    Box(
        contentAlignment = Alignment.Center,
        modifier = Modifier
            .fillMaxSize()
    ){
        Image(
            painter = painterResource(id = R.drawable.splash),
            contentDescription = "Splash",
            modifier = Modifier
                .fillMaxSize()
                .alpha(alpha = alpha),
            contentScale = ContentScale.Crop
        )
    }
}