package com.mmoragues.herois.ui.sharedComponents

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.wrapContentWidth
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import com.mmoragues.herois.ui.theme.Shapes
import com.mmoragues.herois.ui.theme.heroGreenColor
import com.mmoragues.herois.ui.theme.heroOrangeColor
import com.mmoragues.herois.ui.theme.heroRedColor

/**
 * Composable que mostra un tag amb l'alineació (bo, dolent o neutral) del heroi
 * Segons quina sigui l'alineació, el tag tindra un color de fons diferent
 */
@Composable
fun AlignmentTag(alignment: String, modifier: Modifier = Modifier){
    val color = if(alignment == "good") heroGreenColor else if(alignment == "bad") heroRedColor else heroOrangeColor
    ChipView(aligment = alignment, color = color, modifier = modifier)
}

@Composable
fun ChipView(aligment: String, color: Color, modifier: Modifier){
    Box(
        modifier = modifier
            .wrapContentWidth()
            .clip(Shapes.small)
            .background(color)
    ) {
        Text(
            text = aligment,
            modifier = Modifier.padding(12.dp, 6.dp, 12.dp, 6.dp),
            style = MaterialTheme.typography.caption,
            color = Color.White,
            fontWeight = FontWeight.Bold
        )
    }
}