package com.mmoragues.herois.ui.sharedComponents

import androidx.compose.material.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.navigation.NavController
import androidx.navigation.compose.currentBackStackEntryAsState
import com.mmoragues.herois.ui.navigation.BottomMenuOption

/**
 * Composable de la Bottom bar que es farà servir a les pantalles de l'aplicació
 * Es mostren les 4 opcions per navegar a les 4 pantalles de l'aplicació
 * Si es torna enrere amb la fletxa de navegació sempre es torna a la pantalla Home
 * Així s'evita anara guardant a l'historic de pantalles totes les pantalles que hem anat visitant
 */
@Composable
fun BottomBar(navController: NavController){
    val items = listOf(
        BottomMenuOption.Home,
        BottomMenuOption.Favorite,
        BottomMenuOption.Random,
        BottomMenuOption.Game
    )

    BottomNavigation (
        backgroundColor = MaterialTheme.colors.primary,
        contentColor = Color.White
    ){
        val navBackStackEntry by navController.currentBackStackEntryAsState()
        val currentRoute = navBackStackEntry?.destination?.route
        items.forEach { item ->
            BottomNavigationItem(
                icon = { Icon(
                    painterResource(id = item.icon),
                    contentDescription = stringResource(id = item.titleId)
                ) },
                label = {Text(
                    text = stringResource(id = item.titleId),
                    color = MaterialTheme.colors.onPrimary
                )},
                selectedContentColor = MaterialTheme.colors.onPrimary,
                unselectedContentColor = MaterialTheme.colors.onPrimary.copy(0.4f),
                alwaysShowLabel = true,
                selected = currentRoute == item.route,
                onClick = {
                    navController.navigate(item.route){
                        // Fem "Pop up" a la destinació inicial del graf
                        // Així evitem crear una pila molt llarga de destinacions
                        // a mesura que que els usuaris van seleccionant diferents items
                        popUpTo(BottomMenuOption.Home.route)

                        // Evitem multiples copies de la mateixa destinació quan
                        // es reselecciona un mateix item
                        launchSingleTop = true

                        // Restablim l'estat quan es reselecciona un item ja seleccionat anteriorment
                        restoreState = true
                    }
                }
            )
        }
    }

}