package com.mmoragues.herois.ui.sharedComponents

import androidx.compose.foundation.Image
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.rounded.Favorite
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.testTag
import androidx.compose.ui.unit.dp
import coil.compose.rememberImagePainter
import com.mmoragues.herois.R
import com.mmoragues.herois.domain.model.Hero
import com.mmoragues.herois.ui.sharedComponents.AlignmentTag
import com.mmoragues.herois.ui.theme.Shapes

/**
 * Item de la llista d'herois que mostra la carta amb les dades dels herois
 */
@Composable
fun ItemHeroCard(
    hero: Hero,
    onItemClicked: (hero: Hero) -> Unit,
    onFavoriteClicked: (hero: Hero) -> Unit
) {
    Card(
        modifier = Modifier
            .testTag("Card ${hero.name}")
            .fillMaxWidth()
            .padding(8.dp)
            .clip(Shapes.medium)
            .clickable(onClick = { onItemClicked(hero) }),
        elevation = 0.dp,
        backgroundColor = MaterialTheme.colors.surface
    ) {

        Row(
            modifier = Modifier
                .fillMaxWidth()
                .padding(16.dp)
        ) {
            Column(
                modifier = Modifier
                    .weight(1f)
                    .align(Alignment.CenterVertically)
            ) {
                val painter = rememberImagePainter(
                    data = hero.images.sm,
                    builder = {
                        error(R.drawable.offline)
                        crossfade(500)
                    }
                )

                Image(
                    modifier = Modifier
                        .size(80.dp, 80.dp)
                        .clip(RoundedCornerShape(16.dp)),
                    painter = painter,
                    alignment = Alignment.Center,
                    contentDescription = hero.name,
                    contentScale = ContentScale.Crop
                )
            }

            Spacer(modifier = Modifier.width(8.dp))

            Column(
                modifier = Modifier
                    .weight(3f)
                    .align(Alignment.CenterVertically)
            ) {
                Text(
                    text = hero.name,
                    modifier = Modifier.padding(0.dp, 0.dp, 12.dp, 0.dp),
                    color = MaterialTheme.colors.onSurface,
                    style = MaterialTheme.typography.body1//subtitle1
                )

                Spacer(modifier = Modifier.height(8.dp))

                Text(
                    text = hero.biography.fullName,
                    modifier = Modifier.padding(0.dp, 0.dp, 12.dp, 0.dp),
                    color = MaterialTheme.colors.onSurface,
                    style = MaterialTheme.typography.body2
                )
            }

            Column(
                modifier = Modifier
                    .weight(1f)
                    .align(Alignment.CenterVertically)
            ) {
                IconButton(onClick = { onFavoriteClicked(hero) }) {
                    Icon(
                        imageVector = Icons.Rounded.Favorite,
                        contentDescription = "Favorite Icon",
                        tint = if (hero.isFavorite) Color.Red else MaterialTheme.colors.onSurface
                    )
                }
                Spacer(modifier = Modifier.height(15.dp))
                AlignmentTag(alignment = hero.biography.alignment)
            }

        }
    }

}