package com.mmoragues.herois.ui.sharedComponents.heroDetails

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import com.mmoragues.herois.ui.theme.Shapes

/**
 * Composable que mostra les dades d'una característica física de l'heroi (pes, alçada, color ulls, etc.)
 */
@Composable
fun AppearanceCard(
    title: String,
    value: String
){
    Box(
        modifier = Modifier
            .size(100.dp)
            .clip(Shapes.small)
            .background(MaterialTheme.colors.secondary)
            .padding(8.dp),
        contentAlignment = Alignment.Center
    ) {
        Column(
            verticalArrangement = Arrangement.Center,
            modifier = Modifier.wrapContentWidth()
        ) {
            Text(
                text = value,
                modifier = Modifier.fillMaxWidth(),
                color = MaterialTheme.colors.onSecondary,
                style = MaterialTheme.typography.subtitle1,
                fontWeight = FontWeight.W600,
                textAlign = TextAlign.Center
            )

            Spacer(modifier = Modifier.height(4.dp))

            Text(
                text = title,
                modifier = Modifier.fillMaxWidth(),
                color = MaterialTheme.colors.onSecondary.copy(alpha = 0.5f),
                style = MaterialTheme.typography.overline,
                textAlign = TextAlign.Center
            )
        }
    }
}