package com.mmoragues.herois.ui.sharedComponents.heroDetails

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.testTag
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import com.mmoragues.herois.R
import com.mmoragues.herois.common.TestConstants.APPEARANCESECTION
import com.mmoragues.herois.common.TestConstants.BIOGRAPHYSECTION
import com.mmoragues.herois.common.TestConstants.CONNECTIONSECTION
import com.mmoragues.herois.common.TestConstants.WORKSECTION
import com.mmoragues.herois.domain.model.Hero

/**
 * Composable amb els detalls del herois
 * - Imatge
 * - Poders
 * - Biografia
 * - Aparença
 * - Connexions
 * - Treball
 */
@Composable
fun HeroDetails(
    hero: Hero,
    isRandom: Boolean = false,
    paddingValues: PaddingValues,
    onFavoriteClicked: (hero: Hero) -> Unit,
    onRandomClicked: () -> Unit = {}
) {
    LazyColumn(
        modifier = Modifier
            .testTag("LazyColumn")
            .fillMaxSize()
            .background(color = MaterialTheme.colors.surface)
            .padding(10.dp),
        contentPadding = paddingValues
    ) {
        //Imatge
        item {
            ImageSection(
                hero = hero,
                isRandom = isRandom,
                onFavoriteClicked = onFavoriteClicked,
                onRandomClicked = onRandomClicked
            )
            Spacer(modifier = Modifier.height(10.dp))
        }

        //Poders
        item {
            PowerSection(hero = hero)
            Spacer(modifier = Modifier.height(10.dp))
        }

        //Biografia
        item {
            ExpandableCard(
                modifier = Modifier.testTag(BIOGRAPHYSECTION),
                title = stringResource(id = R.string.detail_biography_section),
                description = {
                    BiographySection(hero = hero)
                }
            )
            Spacer(modifier = Modifier.height(10.dp))
        }

        //Apariència
        item {
            ExpandableCard(
                modifier = Modifier.testTag(APPEARANCESECTION),
                title = stringResource(id = R.string.detail_appearance_section),
                description = {
                    AppearanceSection(hero = hero)
                }
            )
            Spacer(modifier = Modifier.height(10.dp))
        }

        //Connexions
        item {
            ExpandableCard(
                modifier = Modifier.testTag(CONNECTIONSECTION),
                title = stringResource(id = R.string.detail_connections_section),
                description = {
                    ConnectionsSection(hero = hero)
                }
            )
            Spacer(modifier = Modifier.height(10.dp))
        }

        //Treball
        item {
            ExpandableCard(
                modifier = Modifier.testTag(WORKSECTION),
                title = stringResource(id = R.string.detail_work_section),
                description = {
                    WorkSection(hero = hero)
                }
            )
            Spacer(modifier = Modifier.height(10.dp))
        }
    }

}