package com.mmoragues.herois.ui.sharedComponents.heroDetails

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.material.Icon
import androidx.compose.material.IconButton
import androidx.compose.material.MaterialTheme
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.rounded.Autorenew
import androidx.compose.material.icons.rounded.Favorite
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.scale
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.testTag
import androidx.compose.ui.semantics.contentDescription
import androidx.compose.ui.semantics.semantics
import androidx.compose.ui.unit.dp
import coil.compose.rememberImagePainter
import com.mmoragues.herois.R
import com.mmoragues.herois.common.TestConstants.IMAGESECTION
import com.mmoragues.herois.domain.model.Hero
import com.mmoragues.herois.ui.sharedComponents.AlignmentTag

/**
 * Secció on es mostra:
 * - Imatge de l'heroi
 * - Alineació (bo, dolent, neutre)
 * - Icona per seleccionar/deseleccionar l'heroi com a favorit
 * - Icona per tornar a demana un nou heroi aleatori (si és que aquesta secció es mostra a la pantalla Random)
 */
@Composable
fun ImageSection(
    hero: Hero,
    isRandom: Boolean,
    onFavoriteClicked: (hero: Hero) -> Unit,
    onRandomClicked: () -> Unit
) {

    val painter = rememberImagePainter(
        data = hero.images.md,
        builder = {
            error(R.drawable.offline)
            crossfade(500)
        }
    )

    Box(
        modifier = Modifier
            .fillMaxWidth()
            .testTag(IMAGESECTION),
        contentAlignment = Alignment.Center
    ) {

        //Imatge de l'heroi
        Image(
            painter = painter,
            alignment = Alignment.CenterStart,
            contentDescription = hero.name,
            contentScale = ContentScale.Crop,
            modifier = Modifier
                .height(300.dp)
                .padding(vertical = 10.dp)
        )

        //Tag amb l'alineació de l'heroi. Es mou a la part superior esquerra de la imatge
        AlignmentTag(
            alignment = hero.biography.alignment,
            modifier = Modifier.offset(x = -100.dp, y = -100.dp)
        )

        //Icona de favorit. Es mou a la part superior detra de la imatge
        IconButton(
            onClick = {
                if (hero != null) {
                    onFavoriteClicked(hero)
                }
            },
            modifier = Modifier
                //.align(Alignment.TopEnd)
                .scale(0.5f)
                .offset(x = 190.dp, y = -200.dp)
                .background(MaterialTheme.colors.primary)

        ) {
            Icon(
                imageVector = Icons.Rounded.Favorite,
                contentDescription = "Favorite Icon",
                modifier = Modifier
                    .align(Alignment.Center)
                    .scale(2f),
                tint = if (hero.isFavorite) Color.Red else MaterialTheme.colors.onPrimary
            )
        }

        //Si estem a la pantalla Random, també mostre el botó de nou Heroi Random
        //Es situa a la part inferior dreta de la imatge
        if (isRandom) {
            IconButton(
                onClick = { onRandomClicked() },
                modifier = Modifier
                    .scale(0.5f)
                    .offset(190.dp, 200.dp)
                    .background(MaterialTheme.colors.primary)
            ) {
                Icon(
                    imageVector = Icons.Rounded.Autorenew,
                    contentDescription = "Random Icon",
                    modifier = Modifier
                        .align(Alignment.Center)
                        .scale(2f),
                    tint = MaterialTheme.colors.onPrimary
                )
            }
        }
    }
}