package com.mmoragues.herois.ui.sharedComponents.powerStat

import androidx.compose.animation.core.animateFloatAsState
import androidx.compose.animation.core.tween
import androidx.compose.foundation.background
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.TextUnit
import androidx.compose.ui.unit.dp

/**
 * Barra animada on es mostra el valor del d'un poder
 * Cada poder té un nom, valor i color diferent que se li passa com a paràmetres
 */
@Composable
fun PowerStatBar(
    powerNameId: Int,
    powerValue: Int,
    powerColor: Color,
    height: Dp = 28.dp,
    animDuration: Int = 1000,
    animDelay: Int = 0,
    fontSize: TextUnit = MaterialTheme.typography.h6.fontSize,
    fontWeight: FontWeight = FontWeight.Bold,
    modifier: Modifier = Modifier
) {

    //Configurem l'animació
    var animationPlayed by rememberSaveable {
        mutableStateOf(false)
    }

    val curPercent = animateFloatAsState(
        targetValue = if (animationPlayed) {
            powerValue / 100f
        } else 0f,
        animationSpec = tween(
            animDuration,
            animDelay
        )
    )

    //Llancem l'animació
    LaunchedEffect(key1 = true) {
        animationPlayed = true
    }

    //Pesos de les columnes
    val column1Weight = .8f // 80%
    val column2Weight = .2f // 20%

    Row() {
        //A la primera columna es mostra la barra animada amb el nom del poder dintre
        Column(modifier = modifier.weight(column1Weight)) {
            Box(
                modifier = modifier
                    .fillMaxWidth()
                    .height(height)
                    .clip(CircleShape)
                    .background(
                        if (isSystemInDarkTheme()) {
                            Color(0xFF505050)
                        } else {
                            Color.LightGray
                        }
                    )
            ){
                //Es va omplint la barra amb l'animació fins al seu valor final
                Row(
                    modifier = modifier
                        .fillMaxHeight()
                        .fillMaxWidth(curPercent.value)
                        .clip(CircleShape)
                        .background(powerColor),
                    content = {}
                )

                Text(text = stringResource(id = powerNameId),
                    modifier = Modifier
                        .align(Alignment.Center),
                    fontWeight = fontWeight,
                    fontSize = fontSize
                )
            }
        }

        //A la segona columna es mostra el valor del poder i com va creixent fins al seu valor final

        Column(
            modifier = modifier.weight(column2Weight),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            Text(
                text = ("${(curPercent.value * 100).toInt()}%"),
                fontSize = fontSize
            )

        }
    }

}