package com.mmoragues.herois.domain.useCase

import com.google.common.truth.Truth.assertThat
import com.mmoragues.herois.common.Resource
import com.mmoragues.herois.data.database.FakeLocalDataSource
import com.mmoragues.herois.data.remote.FakeRemoteDataSource
import com.mmoragues.herois.data.repository.HeroRepositoryImpl
import com.mmoragues.herois.domain.model.Hero
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.runBlocking
import org.junit.Before
import org.junit.Test

//Provem el cas d'ús GetAllHeroesUseCase
class GetAllHeroesUseCaseTest {

    private lateinit var fakeLocalDataSource: FakeLocalDataSource
    private lateinit var fakeRemoteDataSource: FakeRemoteDataSource
    private lateinit var fakeRemoteDataSourceError: FakeRemoteDataSource
    private lateinit var fakeLocalHeroes: MutableList<Hero>
    private lateinit var fakeRemoteHeroes: MutableList<Hero>
    private lateinit var heroRepositoryImpl: HeroRepositoryImpl
    private lateinit var heroRepositoryImplError: HeroRepositoryImpl

    @Before
    fun setUp() {
        fakeLocalDataSource = FakeLocalDataSource()
        fakeRemoteDataSource = FakeRemoteDataSource()
        fakeRemoteDataSourceError = FakeRemoteDataSource(throwException = true)

        fakeLocalHeroes = mutableListOf()
        ('a'..'c').forEachIndexed { index, c ->
            fakeLocalHeroes.add(Hero(id = index, name = c.toString()))
        }
        fakeLocalDataSource.insertFakeHeroes(fakeLocalHeroes)

        fakeRemoteHeroes = mutableListOf()
        ('a'..'z').forEachIndexed { index, c ->
            fakeRemoteHeroes.add(Hero(id = index, name = c.toString()))
        }
        fakeRemoteDataSource.insertFakeHeroes(fakeRemoteHeroes)

        heroRepositoryImpl = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSource)
        heroRepositoryImplError = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSourceError)
    }

    // Comprovem que obtenim els herois quan el servidor remot no té cap problema
    @Test
    fun `Get All Heroes Loading and Success`() = runBlocking {

        val getAllHeroesUseCase = GetAllHeroesUseCase(heroRepositoryImpl)

        getAllHeroesUseCase().collect { result ->
            when (result) {
                is Resource.Loading -> {
                    assertThat(result.data).isEqualTo(fakeLocalHeroes)
                    System.out.println("Loading OK")
                }
                is Resource.Success -> {
                    assertThat(result.data).isEqualTo(fakeRemoteHeroes)
                    System.out.println("Success OK")
                }
                is Resource.Error -> {
                    System.out.println("No hauria d'haver rebut un Resource.Error")
                    assertThat(true).isFalse()
                }
            }
        }
    }

    // Comprovem que obtenim els herois de la cache quan el servidor remot té algún problema
    @Test
    fun `Get All Heroes Loading and Error`() = runBlocking {

        val getAllHeroesUseCase = GetAllHeroesUseCase(heroRepositoryImplError)

        getAllHeroesUseCase().collect { result ->
            when (result) {
                is Resource.Loading -> {
                    assertThat(result.data).isEqualTo(fakeLocalHeroes)
                    System.out.println("Loading OK")
                }
                is Resource.Success -> {
                    System.out.println("No hauria d'haver rebut un Resource.Success")
                    assertThat(true).isFalse()
                }
                is Resource.Error -> {
                    assertThat(result.messageId).isNotNull()
                    assertThat(result.data).isEqualTo(fakeLocalHeroes)
                    System.out.println("Error OK")
                }
            }
        }
    }

}