package com.mmoragues.herois.domain.useCase

import com.google.common.truth.Truth.assertThat
import com.mmoragues.herois.common.Resource
import com.mmoragues.herois.data.database.FakeLocalDataSource
import com.mmoragues.herois.data.remote.FakeRemoteDataSource
import com.mmoragues.herois.data.repository.HeroRepositoryImpl
import com.mmoragues.herois.domain.model.Hero
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.runBlocking
import org.junit.After
import org.junit.Before
import org.junit.Test

//Provem el cas d'ús GetFavoritesHeroesUseCase
class GetFavoritesHeroesUseCaseTest {

    private lateinit var fakeLocalDataSource: FakeLocalDataSource
    private lateinit var fakeRemoteDataSource: FakeRemoteDataSource
    private lateinit var fakeRemoteDataSourceError: FakeRemoteDataSource
    private lateinit var fakeLocalHeroes: MutableList<Hero>
    private lateinit var fakeRemoteHeroes: MutableList<Hero>
    private lateinit var heroRepositoryImpl: HeroRepositoryImpl
    private lateinit var heroRepositoryImplError: HeroRepositoryImpl

    @Before
    fun setUp() {
        fakeLocalDataSource = FakeLocalDataSource()
        fakeRemoteDataSource = FakeRemoteDataSource()
        fakeRemoteDataSourceError = FakeRemoteDataSource(throwException = true)

        fakeLocalHeroes = mutableListOf()
        ('a'..'c').forEachIndexed { index, c ->
            fakeLocalHeroes.add(Hero(id = index, name = c.toString(), isFavorite = true))
        }
        fakeLocalDataSource.insertFakeHeroes(fakeLocalHeroes)

        fakeRemoteHeroes = mutableListOf()
        ('a'..'z').forEachIndexed { index, c ->
            fakeRemoteHeroes.add(Hero(id = index, name = c.toString(), isFavorite = true))
        }
        fakeRemoteDataSource.insertFakeHeroes(fakeRemoteHeroes)

        heroRepositoryImpl = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSource)
        heroRepositoryImplError = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSourceError)
    }

    private suspend fun updateRepository(repository: HeroRepositoryImpl) {
        repository.getAllHeroes().collect { result ->
            when (result) {
                is Resource.Loading -> {
                    System.out.println("Dades en Loading")
                }
                is Resource.Success -> {
                    System.out.println("Dades en Success")
                }
                is Resource.Error -> {
                    System.out.println("Dades en Error")
                }
            }
        }
    }

    //Obtenim el llistat d'usuaris favorits en situació normal
    @Test
    fun `Get Favorites Heroes`() = runBlocking {

        updateRepository(heroRepositoryImpl)
        val getFavoritesHeroesUseCase = GetFavoritesHeroesUseCase(heroRepositoryImpl)


        val heroes = getFavoritesHeroesUseCase().first()
        assertThat(heroes).containsExactlyElementsIn(fakeRemoteHeroes)
        System.out.println("Get Favorites Heroes OK $heroes")
    }

    //Obtenim el llistat d'usuaris favorits sense accés al servidor remot
    @Test
    fun `Get Favorites Heroes without Internet`() = runBlocking {

        updateRepository(heroRepositoryImplError)
        val getFavoritesHeroesUseCase = GetFavoritesHeroesUseCase(heroRepositoryImplError)

        val heroes = getFavoritesHeroesUseCase().first()
        assertThat(heroes).containsExactlyElementsIn(fakeLocalHeroes)
        System.out.println("Get Favorites Heroes sense Internet OK $heroes")
    }

}