package com.mmoragues.herois.domain.useCase

import com.google.common.truth.Truth.assertThat
import com.mmoragues.herois.common.Resource
import com.mmoragues.herois.data.database.FakeLocalDataSource
import com.mmoragues.herois.data.remote.FakeRemoteDataSource
import com.mmoragues.herois.data.repository.HeroRepositoryImpl
import com.mmoragues.herois.domain.model.Hero
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.runBlocking
import org.junit.Before
import org.junit.Test

//Provem el cas d'ús GetHeroByIdUseCase
class GetHeroByIdUseCaseTest{
    private lateinit var fakeLocalDataSource: FakeLocalDataSource
    private lateinit var fakeRemoteDataSource: FakeRemoteDataSource
    private lateinit var fakeRemoteDataSourceError: FakeRemoteDataSource
    private lateinit var fakeLocalHeroes: MutableList<Hero>
    private lateinit var fakeRemoteHeroes: MutableList<Hero>
    private lateinit var heroRepositoryImpl: HeroRepositoryImpl
    private lateinit var heroRepositoryImplError: HeroRepositoryImpl

    @Before
    fun setUp() {
        fakeLocalDataSource = FakeLocalDataSource()
        fakeRemoteDataSource = FakeRemoteDataSource()
        fakeRemoteDataSourceError = FakeRemoteDataSource(throwException = true)

        fakeLocalHeroes = mutableListOf()
        ('a'..'c').forEachIndexed { index, c ->
            fakeLocalHeroes.add(Hero(id = index, name = c.toString()))
        }
        fakeLocalDataSource.insertFakeHeroes(fakeLocalHeroes)

        fakeRemoteHeroes = mutableListOf()
        ('a'..'z').forEachIndexed { index, c ->
            fakeRemoteHeroes.add(Hero(id = index, name = c.toString()))
        }
        fakeRemoteDataSource.insertFakeHeroes(fakeRemoteHeroes)

        heroRepositoryImpl = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSource)
        heroRepositoryImplError = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSourceError)
    }

    private suspend fun updateRepository(repository: HeroRepositoryImpl) {
        repository.getAllHeroes().collect { result ->
            when (result) {
                is Resource.Loading -> {
                    System.out.println("Dades en Loading")
                }
                is Resource.Success -> {
                    System.out.println("Dades en Success")
                }
                is Resource.Error -> {
                    System.out.println("Dades en Error")
                }
            }
        }
    }

    //Obtenim un heroi pel seu ID en situació normal
    @Test
    fun `Get Hero by ID`() = runBlocking {
        updateRepository(heroRepositoryImpl)
        val getHeroByIdUseCase = GetHeroByIdUseCase(heroRepositoryImpl)

        val heroRecuperat = getHeroByIdUseCase(20).first()
        assertThat(heroRecuperat).isEqualTo(fakeRemoteHeroes[20])
        System.out.println("Get a Hero By ID OK. ID: ${heroRecuperat.id}")
    }

    //Obtenim un heroi pel seu ID sense accés al servidor
    @Test
    fun `Get Hero by ID without Internet`() = runBlocking {
        updateRepository(heroRepositoryImplError)
        val getHeroByIdUseCase = GetHeroByIdUseCase(heroRepositoryImplError)

        val heroRecuperat = getHeroByIdUseCase(0).first()
        assertThat(heroRecuperat).isEqualTo(fakeLocalHeroes[0])
        System.out.println("Get a Hero By ID sense Internet OK")
    }

}