package com.mmoragues.herois.domain.useCase

import com.google.common.truth.Truth.assertThat
import com.mmoragues.herois.common.Resource
import com.mmoragues.herois.data.database.FakeLocalDataSource
import com.mmoragues.herois.data.remote.FakeRemoteDataSource
import com.mmoragues.herois.data.repository.HeroRepositoryImpl
import com.mmoragues.herois.domain.model.Hero
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.runBlocking
import org.junit.Before
import org.junit.Test

//Provem el cas d'ús ToggleFavoriteUseCase
class ToggleFavoriteUseCaseTest {
    private lateinit var fakeLocalDataSource: FakeLocalDataSource
    private lateinit var fakeRemoteDataSource: FakeRemoteDataSource
    private lateinit var fakeRemoteDataSourceError: FakeRemoteDataSource
    private lateinit var fakeLocalHeroes: MutableList<Hero>
    private lateinit var fakeRemoteHeroes: MutableList<Hero>
    private lateinit var heroRepositoryImpl: HeroRepositoryImpl
    private lateinit var heroRepositoryImplError: HeroRepositoryImpl

    @Before
    fun setUp() {
        fakeLocalDataSource = FakeLocalDataSource()
        fakeRemoteDataSource = FakeRemoteDataSource()
        fakeRemoteDataSourceError = FakeRemoteDataSource(throwException = true)

        fakeLocalHeroes = mutableListOf()
        ('a'..'c').forEachIndexed { index, c ->
            fakeLocalHeroes.add(Hero(id = index, name = c.toString(), isFavorite = (index % 2) == 1))
        }
        fakeLocalDataSource.insertFakeHeroes(fakeLocalHeroes)

        fakeRemoteHeroes = mutableListOf()
        ('a'..'z').forEachIndexed { index, c ->
            fakeRemoteHeroes.add(Hero(id = index, name = c.toString()))
        }
        fakeRemoteDataSource.insertFakeHeroes(fakeRemoteHeroes)

        heroRepositoryImpl = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSource)
        heroRepositoryImplError = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSourceError)
    }

    // Afegim un heroi a la llista de favorits
    @Test
    fun `Toggle isFavorite from False to True`() = runBlocking {
        val toggleFavoriteUseCase = ToggleFavoriteUseCase(heroRepositoryImpl)
        toggleFavoriteUseCase(fakeLocalHeroes[0])
        val hero = heroRepositoryImpl.getHeroById(0).first()
        assertThat(hero.isFavorite).isTrue()
        System.out.println("Toggle from False to True OK. Nou valor isFavorite: ${hero.isFavorite}")
    }

    // Treiem un heroi de la llista de favorits
    @Test
    fun `Toggle isFavorite from True to False`() = runBlocking {
        val toggleFavoriteUseCase = ToggleFavoriteUseCase(heroRepositoryImpl)
        toggleFavoriteUseCase(fakeLocalHeroes[1])
        val hero = heroRepositoryImpl.getHeroById(1).first()
        assertThat(hero.isFavorite).isFalse()
        System.out.println("Toggle from False to True OK. Nou valor isFavorite: ${hero.isFavorite}")
    }

}